import sagemaker, boto3
region = sagemaker.Session().boto_region_name

print("SageMaker setup program")
print("This program will setup a SageMaker model that can be used to try out QuickSight's SageMaker integration.")
print("----------------------------------------------------------------------------------------------------------")
print("You are running this program in " + region)

role = sagemaker.get_execution_role()
print("Role being used currently is " + role)
print("This role should have admin/elevated rights to be able to create S3 bucket and SageMaker model.")
print("----------------------------------------------------------------------------------------------------------")


awsAccountId = role.split(':')[4]

autoMlDockerAccount=sagemaker.image_uris.retrieve("xgboost", region, "1.2-2").split('.')[0]

sklearnContainer = autoMlDockerAccount + '.dkr.ecr.' + region + '.amazonaws.com/sagemaker-sklearn-automl:0.2-1-cpu-py3'
xgboostContainer = autoMlDockerAccount + '.dkr.ecr.' + region + '.amazonaws.com/sagemaker-xgboost:1.2-2-cpu-py3'

bucketName = 'quicksight-sagemaker-'+str(awsAccountId)+'-'+region
modelName = 'QSWS-OrderCompletionSLAPrediction'

try:
    s3=boto3.client('s3',region_name=region)
    print('About to create bucket ' + bucketName)
    if region != 'us-east-1':
        s3.create_bucket(
            ACL='private',
            CreateBucketConfiguration={
                'LocationConstraint': region
            },
            Bucket = bucketName )
    else:
        s3.create_bucket(
            ACL='private',
            Bucket = bucketName )

except s3.exceptions.BucketAlreadyOwnedByYou:
    print('Bucket ' + bucketName + ' already exists - Moving on to next step')
except Exception as e:
    print("----------------------------------------------------------------------------------------------------------")
    print('ERROR')
    print(e)
    print('Can not proceed further. Exiting this program. Please troubleshoot based on above error.')
    print("----------------------------------------------------------------------------------------------------------")
    exit()

sagemaker.s3.S3Downloader.download('s3://ee-assets-prod-us-east-1/modules/426df837a1774b49be27075dc6d21221/v1/order-completion-sla-data-processor-model.tar.gz','./SageMakerModels')
sagemaker.s3.S3Downloader.download('s3://ee-assets-prod-us-east-1/modules/426df837a1774b49be27075dc6d21221/v1/order-completion-sla-tuning-model.tar.gz','./SageMakerModels')

sagemaker.s3.S3Uploader.upload('./SageMakerModels','s3://' + bucketName + '/models')



sm = boto3.client('sagemaker', region_name = region)

try:
    response = sm.create_model(ModelName='QSWS-OrderCompletionSLAPrediction',
        Containers=[
            {
                'Image': sklearnContainer,
                'Mode': 'SingleModel',
                'ModelDataUrl': 's3://' + bucketName + '/models/order-completion-sla-data-processor-model.tar.gz',
                'Environment': {
                    "AUTOML_SPARSE_ENCODE_RECORDIO_PROTOBUF": "1",
                    "AUTOML_TRANSFORM_MODE": "feature-transform",
                    "SAGEMAKER_DEFAULT_INVOCATIONS_ACCEPT": "application/x-recordio-protobuf",
                    "SAGEMAKER_PROGRAM": "sagemaker_serve",
                    "SAGEMAKER_SUBMIT_DIRECTORY": "/opt/ml/model/code"
                }
            },
            {
                'Image': xgboostContainer,
                'Mode': 'SingleModel',
                'ModelDataUrl': 's3://' + bucketName + '/models/order-completion-sla-tuning-model.tar.gz',
                'Environment': {
                    "MAX_CONTENT_LENGTH": "20971520",
                    "SAGEMAKER_DEFAULT_INVOCATIONS_ACCEPT": "text/csv",
                    "SAGEMAKER_INFERENCE_OUTPUT": "predicted_label",
                    "SAGEMAKER_INFERENCE_SUPPORTED": "predicted_label,probability,probabilities"
                }
            },
            {
                'Image': sklearnContainer,
                'Mode': 'SingleModel',
                'ModelDataUrl': 's3://' + bucketName + '/models/order-completion-sla-data-processor-model.tar.gz',
                'Environment': {
                    "AUTOML_TRANSFORM_MODE": "inverse-label-transform",
                    "SAGEMAKER_DEFAULT_INVOCATIONS_ACCEPT": "text/csv",
                    "SAGEMAKER_INFERENCE_INPUT": "predicted_label",
                    "SAGEMAKER_INFERENCE_OUTPUT": "predicted_label",
                    "SAGEMAKER_INFERENCE_SUPPORTED": "predicted_label,probability,labels,probabilities",
                    "SAGEMAKER_PROGRAM": "sagemaker_serve",
                    "SAGEMAKER_SUBMIT_DIRECTORY": "/opt/ml/model/code"
                }
            }
            ],
            InferenceExecutionConfig={
                'Mode': 'Serial'
            }  ,
            ExecutionRoleArn='arn:aws:iam::' + awsAccountId + ':role/SageMakerRoleForQuickSightWorkshop',
            EnableNetworkIsolation=False )

    print('Model ' + modelName + ' created. Move on to next step as mentioned in workshop.')

except Exception as e:
    if 'Cannot create already existing model' in str(e):
        print('Model ' + modelName + 'already exists. Move on to next step as mentioned in workshop.')
    else:
        print("----------------------------------------------------------------------------------------------------------")
        print('ERROR')
        print(e)
        print(type(e))
        print('Can not proceed further. Exiting this program. Please troubleshoot based on above error.')
        print("----------------------------------------------------------------------------------------------------------")
        exit()
