#AAI=`aws sts get-caller-identity|grep Account|cut -d":" -f2|cut -d'"' -f2`
#echo "AAI="$AAI >> ~/.bash_profile
#echo "export AAI" >> ~/.bash_profile
#----------------------------------------------------------------------------------------------------------
#Functions
#----------------------------------------------------------------------------------------------------------
#Check if a variable has value.
ValueCheck()
{

    if [ `echo $2|wc -c` -le 1 ]
    then
        echo ----------------------------------------------------------------------------------------------------------
        echo ERROR - $1 is empty
        echo ----------------------------------------------------------------------------------------------------------
        exit 1
    fi

    if [ `echo $3|wc -c` -gt 1 ]
    then
        if [ $2 != $3 ]
        then
            echo ----------------------------------------------------------------------------------------------------------
            echo ERROR - $1 has value $2. Expected value is $3
            echo ----------------------------------------------------------------------------------------------------------
            exit 1
        fi
    fi
}

#----------------------------------------------------------------------------------------------------------
#Main Program
#----------------------------------------------------------------------------------------------------------

#Install jq to help parse more complex JSON responses later on.
echo ----------------------------------------------------------------------------------------------------------
echo QUICKSIGHT ADMIN WORKSHOP
echo Installing jq to help parse more complex JSON responses later on. Press y when prompted.
echo ----------------------------------------------------------------------------------------------------------
sudo yum install jq

echo ----------------------------------------------------------------------------------------------------------
echo Installing boto3.
echo ----------------------------------------------------------------------------------------------------------
pip install boto3

echo ----------------------------------------------------------------------------------------------------------
echo Installing sagemaker python library.
echo ----------------------------------------------------------------------------------------------------------
pip install sagemaker
echo ----------------------------------------------------------------------------------------------------------
echo OUTPUT - Use below info in input to CloudFormation in next step.
echo ----------------------------------------------------------------------------------------------------------

#Get current user identity
#ARN=`aws sts get-caller-identity|grep Arn|cut -d":" -f2- |cut -d'"' -f2`
ARN=`aws sts get-caller-identity|jq .Arn|sed 's/"//g'`
echo Current identity arn : $ARN

ValueCheck 'ARN' $ARN

#Extract AWS Account id from above ARN
AAI=`echo $ARN|cut -d":" -f5`
echo AWS Account Id : $AAI

ValueCheck 'AAI' $AAI

#If Arn is for an assumed role like arn:aws:sts::8408371xxxxx:assumed-role/TeamRole/MasterKey, we want to grab TeamRole/MasterKey as UserName.
#If Arn is for an IAM user like arn:aws:iam::840837168571:user/TempTestUser, we want to grab TempTestUser as the UserName.
#Below statement works for both scenarios.
UserName=`echo $ARN|cut -d"/" -f2-`
echo Corresponding QuickSight UserName : $UserName

ValueCheck 'UserName' $UserName

#Check if user exists in QuickSight. Try running against current region.
aws quicksight describe-user --aws-account-id $AAI --namespace default --user-name $UserName > ~/environment/DescribeUserOutput.json 2>&1
DescribeUserResponse=$?

if [ $DescribeUserResponse -ne 0 ]
then
    #If error is encountered, check if it is due to Cloud9 being in a region different from QuickSight identity region.
    ErrorTypeCheck=`cat ~/environment/DescribeUserOutput.json|grep "Operation is being called from endpoint"|wc -l`
    if [ $ErrorTypeCheck -eq 1 ]
    then
        #If so, parse out QuickSight identity region from the error message
        IdentityRegion=`cat ~/environment/DescribeUserOutput.json|cut -d"." -f2|cut -d" " -f5`
        #Rerun the describe-user command against identity region
        aws quicksight describe-user --aws-account-id $AAI --namespace default --user-name $UserName --region $IdentityRegion> ~/environment/DescribeUserOutput.json 2>&1
    else
        cat ~/environment/DescribeUserOutput.json
        exit 0
    fi
fi
UserResponse=`cat ~/environment/DescribeUserOutput.json`


#Get corresponding QuickSight user's role
#UserRole=`echo $UserResponse|sed 's/,/\n/g'|grep '"Role":'|cut -d'"' -f4`
UserRole=`echo $UserResponse|jq .User.Role|sed 's/"//g'`
echo QuickSight user role : $UserRole

ValueCheck 'UserRole' $UserRole 'ADMIN'

#Get QuickSight identity region
IdentityRegion=`echo $UserResponse|jq .User.Arn|cut -d":" -f4`
echo QuickSight IdentityRegion : $IdentityRegion

ValueCheck 'IdentityRegion' $IdentityRegion

#Get user email
Email=`echo $UserResponse|jq ' .User.Email'|sed 's/"//g'`


#Updating bash profile with variables we need for this workshop
echo "AAI="$AAI >> ~/.bash_profile
echo "IR="$IdentityRegion >> ~/.bash_profile
echo "UN="$UserName >> ~/.bash_profile
echo "EML="$Email >> ~/.bash_profile
echo "export AAI IR UN EML" >> ~/.bash_profile
